#manager {
    margin:8px 12px 12px 12px;
}

#tbtoolbar {
    border:1px solid darkgrey;
    background-color: #ffffff; 
    padding: 0; 
    margin:0 0 12px 0;
}

#tbtoolbar vbox  {
    margin: 0;
    padding:6px 6px 2px 6px;
    background-color: #ffffff;
}

#tbtoolbar vbox:hover  {
    background-color: #e0e8f6;
}

#tbtoolbar vbox[active="true"]  {
    background-color: #c1d2ee;
}

#mainframe {
    padding:0 12px;
    margin:0;
}

.row vbox {
    width:140px;
}

iframe {
    margin:0;
    padding:0;
}
