/*
 * This file is part of TbSync.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. 
 */
 
 "use strict";

var { TbSync } = ChromeUtils.import("chrome://tbsync/content/tbsync.jsm");

var tbSyncMessenger = {

  onInject: function (window) {
    Services.obs.notifyObservers(null, "tbsync.observer.manager.updateSyncstate", null);
  },

  onRemove: function (window) {
  },

};
