treechildren::-moz-tree-image(DirCol, orphaned) {
  margin-inline-end: 2px;
  list-style-image: url("chrome://tbsync/content/skin/error16.png");
}

.abMenuItem[AddrBook="true"][TbSyncIcon="orphaned"] {
  list-style-image: url("chrome://tbsync/content/skin/error16.png");
}
